# -*- coding: utf-8 -*-
"""
Visualization utilities for Automar web application.

This module provides helper functions for date calculations and prediction formatting.
All web visualizations are rendered client-side using Plotly.js in the frontend
(see frontend/src/lib/components/visualizations/).
"""
from . import auxiliary

__all__ = ["auxiliary"]
