import"./DsnmJJEf.js";import{i as t}from"./nYX6uOL5.js";import{p,i as g}from"./CiiADubH.js";import{p as r}from"./BPOCImrP.js";import{S as c,C as h}from"./C0vfx6Fs.js";const d={name:"toml",startState:function(){return{inString:!1,stringType:"",lhs:!0,inArray:0}},token:function(e,n){let i;if(!n.inString&&(i=e.match(/^('''|"""|'|")/))&&(n.stringType=i[0],n.inString=!0),e.sol()&&!n.inString&&n.inArray===0&&(n.lhs=!0),n.inString){for(;n.inString;)if(e.match(n.stringType))n.inString=!1;else if(e.peek()==="\\")e.next(),e.next();else{if(e.eol())break;e.match(/^.[^\\\"\']*/)}return n.lhs?"property":"string"}else{if(n.inArray&&e.peek()==="]")return e.next(),n.inArray--,"bracket";if(n.lhs&&e.peek()==="["&&e.skipTo("]"))return e.next(),e.peek()==="]"&&e.next(),"atom";if(e.peek()==="#")return e.skipToEnd(),"comment";if(e.eatSpace())return null;if(n.lhs&&e.eatWhile(function(l){return l!="="&&l!=" "}))return"property";if(n.lhs&&e.peek()==="=")return e.next(),n.lhs=!1,null;if(!n.lhs&&e.match(/^\d\d\d\d[\d\-\:\.T]*Z/))return"atom";if(!n.lhs&&(e.match("true")||e.match("false")))return"atom";if(!n.lhs&&e.peek()==="[")return n.inArray++,e.next(),"bracket";if(!n.lhs&&e.match(/^\-?\d+(?:\.\d+)?/))return"number";e.eatSpace()||e.next()}return null},languageData:{commentTokens:{line:"#"}}};function x(e,n){p(n,!1);let i=r(n,"value",12,""),l=r(n,"placeholder",8,"TOML configuration..."),o=r(n,"readonly",8,!1);const u=c.define(d);t(),h(e,{get language(){return u},get placeholder(){return l()},get readonly(){return o()},ariaLabel:"TOML configuration editor",get value(){return i()},set value(f){i(f)},$$legacy:!0}),g()}export{x as default};
