import"../chunks/DsnmJJEf.js";import{i as u}from"../chunks/nYX6uOL5.js";import{p as h,j as g,k as l,t as v,l as d,i as _,m as e,n as a,s as x,o}from"../chunks/CiiADubH.js";import{s as k,p}from"../chunks/DBsyeOHX.js";const $={get error(){return p.error},get status(){return p.status}};k.updated.check;const i=$;var b=g("<h1> </h1> <p> </p>",1);function y(m,n){h(n,!1),u();var r=b(),t=l(r),c=e(t,!0);a(t);var s=x(t,2),f=e(s,!0);a(s),v(()=>{o(c,i.status),o(f,i.error?.message)}),d(m,r),_()}export{y as component};
