import{q as p,v as k,w as _,E as S,x as g,y as w,z as E,A as P,B as A,C,D as T,F as L,b as N,G as R,S as $}from"./CiiADubH.js";function B(n,a,i){p&&k();var l=n,c,e,t=null,s=null;function u(){e&&(C(e),e=null),t&&(t.lastChild.remove(),l.before(t),t=null),e=s,s=null}_(()=>{if(c!==(c=a())){var m=P();if(c){var r=l;m&&(t=document.createDocumentFragment(),t.append(r=g()),e&&E.skipped_effects.add(e)),s=w(()=>i(r,c))}m?E.add_callback(u):u()}},S),p&&(l=A)}function b(n,a){return n===a||n?.[$]===a}function j(n={},a,i,l){return T(()=>{var c,e;return L(()=>{c=e,e=[],N(()=>{n!==i(...e)&&(a(n,...e),c&&b(i(...c),n)&&a(null,...c))})}),()=>{R(()=>{e&&b(i(...e),n)&&a(null,...e)})}}),n}const q="modulepreload",x=function(n,a){return new URL(n,a).href},y={},D=function(a,i,l){let c=Promise.resolve();if(i&&i.length>0){let m=function(r){return Promise.all(r.map(f=>Promise.resolve(f).then(d=>({status:"fulfilled",value:d}),d=>({status:"rejected",reason:d}))))};const t=document.getElementsByTagName("link"),s=document.querySelector("meta[property=csp-nonce]"),u=s?.nonce||s?.getAttribute("nonce");c=m(i.map(r=>{if(r=x(r,l),r in y)return;y[r]=!0;const f=r.endsWith(".css"),d=f?'[rel="stylesheet"]':"";if(l)for(let h=t.length-1;h>=0;h--){const v=t[h];if(v.href===r&&(!f||v.rel==="stylesheet"))return}else if(document.querySelector(`link[href="${r}"]${d}`))return;const o=document.createElement("link");if(o.rel=f?"stylesheet":q,f||(o.as="script"),o.crossOrigin="",o.href=r,u&&o.setAttribute("nonce",u),document.head.appendChild(o),f)return new Promise((h,v)=>{o.addEventListener("load",h),o.addEventListener("error",()=>v(new Error(`Unable to preload CSS for ${r}`)))})}))}function e(t){const s=new Event("vite:preloadError",{cancelable:!0});if(s.payload=t,window.dispatchEvent(s),!s.defaultPrevented)throw t}return c.then(t=>{for(const s of t||[])s.status==="rejected"&&e(s.reason);return a().catch(e)})};export{D as _,j as b,B as c};
