.code-editor-wrapper.svelte-18lj5km{width:100%;height:500px;border:1px solid var(--border-default);border-radius:4px;overflow:hidden}.cm-editor{height:100%;font-size:13px}.cm-scroller{overflow:auto;font-family:Consolas,Monaco,Courier New,monospace}.cm-gutters{background-color:var(--code-bg);border-right:1px solid var(--border-default)}.cm-activeLineGutter{background-color:var(--bg-elevated)}.cm-content{padding:.5rem 0}.cm-selectionBackground{background-color:var(--interactive-primary);opacity:.3}.cm-line{line-height:1.6}
